local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("notifications", "AceEvent-3.0", "AceHook-3.0", "LibSink-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("tomQuest2")
local defaults = {
	profile = {
		playerObjectiveProgress = true,
		playerObjectiveComplete = true,
		playerQuestComplete = true,
		playerQuestFailed = true,
		partyObjectiveProgress = true,
		partyObjectiveComplete = true,
		partyQuestComplete = true,
		partyQuestFailed = true,
		hideBlizzardNotification = true,
		sinkOptions = {
		},
	},
}

local prdb
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("notifications", defaults, "Default")
	prdb = self.db.profile
	self:SetSinkStorage(prdb.sinkOptions)
end

module.modName = L["Quests Notifications"]

local options
function module:getOptions()
	if not options then
		options = {
			hideBlizzardNotification = {
				name = L["Toggle hide default blizzard quests notification"],
				type = "toggle",
				get = function(info)
					return prdb.hideBlizzardNotification
				end,
				set = function(info, value)
					prdb.hideBlizzardNotification = value
				end,
				width = "full",
				order = 10,
			},
			playerheader = {
				type = "header",
				name = L["Player Notifications"],
				order = 11,
			},
			playerObjectiveProgress = {
				name = L["Toggle player quests objectives progress notification"],
				type = "toggle",
				get = function(info)
					return prdb.playerObjectiveProgress
				end,
				set = function(info, value)
					prdb.playerObjectiveProgress = value
				end,
				width = "full",
				order = 12,
			},
			playerObjectiveComplete = {
				name = L["Toggle player quests objectives complete notification"],
				type = "toggle",
				get = function(info)
					return prdb.playerObjectiveComplete
				end,
				set = function(info, value)
					prdb.playerObjectiveComplete = value
				end,
				width = "full",
				order = 13,
			},
			playerQuestComplete = {
				name = L["Toggle player quests complete notification"],
				type = "toggle",
				get = function(info)
					return prdb.playerQuestComplete
				end,
				set = function(info, value)
					prdb.playerQuestComplete = value
				end,
				width = "full",
				order = 14,
			},
			playerQuestFailed = {
				name = L["Toggle player quests failed notification"],
				type = "toggle",
				get = function(info)
					return prdb.playerQuestFailed
				end,
				set = function(info, value)
					prdb.playerQuestFailed = value
				end,
				width = "full",
				order = 15,
			},
			partyheader = {
				type = "header",
				name = L["Party Notifications"],
				order = 16,
			},
			partyObjectiveProgress = {
				name = L["Toggle party quests objectives progress notification"],
				type = "toggle",
				get = function(info)
					return prdb.partyObjectiveProgress
				end,
				set = function(info, value)
					prdb.partyObjectiveProgress = value
				end,
				width = "full",
				order = 17,
			},
			partyObjectiveComplete = {
				name = L["Toggle party quests objectives complete notification"],
				type = "toggle",
				get = function(info)
					return prdb.partyObjectiveComplete
				end,
				set = function(info, value)
					prdb.partyObjectiveComplete = value
				end,
				width = "full",
				order = 18,
			},
			partyQuestComplete = {
				name = L["Toggle party quests complete notification"],
				type = "toggle",
				get = function(info)
					return prdb.partyQuestComplete
				end,
				set = function(info, value)
					prdb.partyQuestComplete = value
				end,
				width = "full",
				order = 16,
			},
			partyQuestFailed = {
				name = L["Toggle party quests failed notification"],
				type = "toggle",
				get = function(info)
					return prdb.partyQuestFailed
				end,
				set = function(info, value)
					prdb.partyQuestFailed = value
				end,
				width = "full",
				order = 19,
			},

		}
		options.output = self:GetSinkAce3OptionsDataTable()
	end
	return options 
end

function module:OnEnable()
	self:RawHook("UIErrorsFrame_OnEvent", true)
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestComplete")
	core.RegisterCallback(self, "playerQuestFailed")
	core.RegisterCallback(self, "partyQuestObjectiveProgress")
	core.RegisterCallback(self, "partyQuestComplete")
	core.RegisterCallback(self, "partyQuestFailed")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

local found = '^'..ERR_QUEST_ADD_FOUND_SII:gsub('(%%.)', '(.+)')..'$' -- "%s: %d/%d"
local item = '^'..ERR_QUEST_ADD_ITEM_SII:gsub('(%%.)', '(.+)')..'$' -- "%s: %d/%d"
local kill = '^'..ERR_QUEST_ADD_KILL_SII:gsub('(%%.)', '(.+)')..'$' -- "%s slain: %d/%d"
local complete = '^'..ERR_QUEST_OBJECTIVE_COMPLETE_S:gsub('(%%.)', '(.+)')..'$' -- "%s (Complete)"

function module:UIErrorsFrame_OnEvent(frame, event, msg, ...)
	if event == "UI_INFO_MESSAGE" and prdb.hideBlizzardNotification and (msg:match(found) or msg:match(item) or msg:match(kill) or msg:match(complete)) then return end
	return self.hooks.UIErrorsFrame_OnEvent(frame, event, msg, ...)
end

function module:playerQuestObjectiveProgress(event, firstRun, commHandler, uid, objective, got, ngot)
	if ngot == core.questsObjectives[uid][objective].nneed and prdb.playerObjectiveComplete and not firstRun then
		module:Pour(format(L["(%s) %s : finished"], core.quests[uid].title, core.questsObjectives[uid][objective].desc), 0, 1, 0)
	elseif prdb.playerObjectiveProgress and not firstRun and ngot > 0 then
		module:Pour(format("(%s) %s : %s/%s", core.quests[uid].title, core.questsObjectives[uid][objective].desc , got, core.questsObjectives[uid][objective].need), core:getColorFromCompletionRGB(ngot/core.questsObjectives[uid][objective].nneed))
	end
end

function module:playerQuestComplete(event, firstRun, commHandler, uid)
	if prdb.playerQuestComplete and not firstRun then
		module:Pour(format(L["(%s) : finished"], core.quests[uid].title), 0, 1, 0)
	end
end

function module:playerQuestFailed(event, firstRun, commHandler, uid)
	if prdb.playerQuestFailed and not firstRun then
		module:Pour(format(L["(%s) : failed"], core.quests[uid].title), 1, 0, 0)
	end
end

function module:partyQuestObjectiveProgress(event, sender, uid, objective, got, ngot)
	if ngot == core.quests[uid].nneed and prdb.partyObjectiveComplete then
		module:Pour(format(L["(%s : %s) %s : finished"], sender, core.quests[uid].title, core.questsObjectives[uid][objective].desc), 0, 1, 0)
	elseif prdb.partyObjectiveProgress then
		module:Pour(format("(%s : %s) %s : %s/%s", sender, core.quests[uid].title, core.questsObjectives[uid][objective].desc , got, core.questsObjectives[uid][objective].need), core:getColorFromCompletionRGB(ngot/core.questsObjectives[uid][objective].nneed))
	end
end

function module:partyQuestComplete(event, sender, uid)
	if prdb.partyQuestComplete then
		module:Pour(format(L["(%s : %s) : finished"], sender, core.quests[uid].title), 0, 1, 0)
	end
end

function module:partyQuestFailed(event, sender, uid)
	if prdb.partyQuestFailed then
		module:Pour(format(L["(%s : %s) : failed"], sender, core.quests[uid].title), 1, 0, 0)
	end
end